'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (component) {
    return function (_component) {
        _inherits(_class, _component);

        function _class() {
            _classCallCheck(this, _class);

            return _possibleConstructorReturn(this, _component.apply(this, arguments));
        }

        /*
         * This needs to be overloaded to return an object
         * representing the current query string, e.g.
         * /page?foo=bar&page=1 should return
         * {foo: bar, page: 1}
         */
        _class.prototype.getQuery = function getQuery() {
            throw new Error('This needs to be overloaded');
        };

        /*
         * This needs to be overloaded to take a query object
         * and replace the current query string with it.
         *
         * In this context, replace means replace the last entry
         * in history, not add a new one.
         *
         * This is used for deleting default query parameters from the
         * query string, so if a user is linked to /foo?page=1 they are
         * replaced to /foo.  The user should not be able to get back
         * to the page=1 query string with their back button.
         */


        _class.prototype.replaceQuery = function replaceQuery() {
            // eslint-disable-line react/sort-comp
            throw new Error('This needs to be overloaded');
        };

        /*
         * This needs to be overloaded to take a query object
         * and push it onto the history stack.
         */


        _class.prototype.pushQuery = function pushQuery() {
            throw new Error('This needs to be overloaded');
        };

        /*
         * Child classes need to implement this to provide the default
         * state values for variables from the query string.  This
         * values will always be available, whether the query string is
         * specified or not.
         *
         * These values get maintained in this.state.qstate
         */


        _class.prototype.getDefaultQuery = function getDefaultQuery() {
            throw new Error('This needs to be overloaded');
        };

        _class.prototype.componentWillMount = function componentWillMount() {
            var _this2 = this;

            var defaults = this.getDefaultQuery();
            var query = this.getQuery();

            this.setState({ qstate: _extends({}, defaults, query) }, function () {
                if (Object.keys(query).filter(function (k) {
                    return defaults[k] === query[k];
                }).length > 0) {
                    _this2.replaceQuery(_this2._cleanupQueryString(_this2.state.qstate));
                }
            });

            if (typeof _component.prototype.componentWillMount === 'function') {
                _component.prototype.componentWillMount.call(this);
            }
        };

        _class.prototype.setQState = function setQState(newState, callback) {
            var _this3 = this;

            var qstate = this._extractQState(newState);

            var _callback = function _callback() {
                _this3.pushQuery(_this3._cleanupQueryString(qstate));

                if (typeof callback === 'function') {
                    callback.call(_this3);
                }
            };

            this.setState({ qstate: qstate }, _callback);
        };

        _class.prototype._extractQState = function _extractQState(newState) {
            // 😎
            return _extends({}, this.getDefaultQuery(), this.state.qstate, typeof newState === 'function' ? newState.call(this, this.state.qstate, this.props) : newState);
        };

        _class.prototype._cleanupQueryString = function _cleanupQueryString(query) {
            var defaults = this.getDefaultQuery();
            Object.keys(query).filter(function (key) {
                return query[key] === defaults[key];
            }).forEach(function (key) {
                return delete query[key];
            });
            return query;
        };

        return _class;
    }(component);
};

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }